package tcp

import (
	"os"
	"strings"

	"gitlab.dms3.io/p2p/go-reuseport"
)

// envReuseport is the env variable name used to turn off reuse port.
// It default to true.
const envReuseport = "P2P_TCP_REUSEPORT"
const deprecatedEnvReuseport = "DMS3_REUSEPORT"

// envReuseportVal stores the value of envReuseport. defaults to true.
var envReuseportVal = true

func init() {
	v := strings.ToLower(os.Getenv(envReuseport))
	if v == "false" || v == "f" || v == "0" {
		envReuseportVal = false
		log.Infof("REUSEPORT disabled (P2P_TCP_REUSEPORT=%s)", v)
	}
	v, exist := os.LookupEnv(deprecatedEnvReuseport)
	if exist {
		log.Warn("DMS3_REUSEPORT is deprecated, use P2P_TCP_REUSEPORT instead")
		if v == "false" || v == "f" || v == "0" {
			envReuseportVal = false
			log.Infof("REUSEPORT disabled (DMS3_REUSEPORT=%s)", v)
		}
	}
}

// reuseportIsAvailable returns whether reuseport is available to be used. This
// is here because we want to be able to turn reuseport on and off selectively.
// For now we use an ENV variable, as this handles our pressing need:
//
//   P2P_TCP_REUSEPORT=false dms3 daemon
//
// If this becomes a sought after feature, we could add this to the config.
// In the end, reuseport is a stop-gap.
func ReuseportIsAvailable() bool {
	return envReuseportVal && reuseport.Available()
}
