package tcp

import (
	"testing"

	"gitlab.dms3.io/p2p/go-p2p-core/crypto"
	"gitlab.dms3.io/p2p/go-p2p-core/peer"

	csms "gitlab.dms3.io/p2p/go-conn-security-multistream"
	sec "gitlab.dms3.io/p2p/go-p2p-core/sec"
	"gitlab.dms3.io/p2p/go-p2p-core/sec/insecure"
	mplex "gitlab.dms3.io/p2p/go-p2p-mplex"
	tptu "gitlab.dms3.io/p2p/go-p2p-transport-upgrader"

	ttransport "gitlab.dms3.io/p2p/go-p2p-testing/suites/transport"

	ma "gitlab.dms3.io/mf/go-multiaddr"
)

func TestTcpTransport(t *testing.T) {
	for i := 0; i < 2; i++ {
		ia, ida := makeInsecureTransport(t)
		ib, _ := makeInsecureTransport(t)

		ta := NewTCPTransport(&tptu.Upgrader{
			Secure: ia,
			Muxer:  new(mplex.Transport),
		})
		tb := NewTCPTransport(&tptu.Upgrader{
			Secure: ib,
			Muxer:  new(mplex.Transport),
		})

		zero := "/ip4/127.0.0.1/tcp/0"
		ttransport.SubtestTransport(t, ta, tb, zero, ida)

		envReuseportVal = false
	}
	envReuseportVal = true
}

func TestTcpTransportCantDialDNS(t *testing.T) {
	for i := 0; i < 2; i++ {
		dnsa, err := ma.NewMultiaddr("/dns4/example.com/tcp/1234")
		if err != nil {
			t.Fatal(err)
		}

		ix, _ := makeInsecureTransport(t)
		tpt := NewTCPTransport(&tptu.Upgrader{
			Secure: ix,
			Muxer:  new(mplex.Transport),
		})

		if tpt.CanDial(dnsa) {
			t.Fatal("shouldn't be able to dial dns")
		}

		envReuseportVal = false
	}
	envReuseportVal = true
}

func TestTcpTransportCantListenUtp(t *testing.T) {
	for i := 0; i < 2; i++ {
		utpa, err := ma.NewMultiaddr("/ip4/127.0.0.1/udp/0/utp")
		if err != nil {
			t.Fatal(err)
		}

		ix, _ := makeInsecureTransport(t)
		tpt := NewTCPTransport(&tptu.Upgrader{
			Secure: ix,
			Muxer:  new(mplex.Transport),
		})

		_, err = tpt.Listen(utpa)
		if err == nil {
			t.Fatal("shouldnt be able to listen on utp addr with tcp transport")
		}

		envReuseportVal = false
	}
	envReuseportVal = true
}

func makeInsecureTransport(t *testing.T) (sec.SecureMuxer, peer.ID) {
	priv, pub, err := crypto.GenerateKeyPair(crypto.Ed25519, 256)
	if err != nil {
		t.Fatal(err)
	}
	id, err := peer.IDFromPublicKey(pub)
	if err != nil {
		t.Fatal(err)
	}
	secMuxer := new(csms.SSMuxer)
	secMuxer.AddTransport(insecure.ID, insecure.NewWithIdentity(id, priv))
	return secMuxer, id
}
